﻿#ifndef SERVICEPUBLISHER_H
#define SERVICEPUBLISHER_H

/*!
 *@file ServicePublisher.h
 *@author hxcan <caihuosheng@gmail.com>
 *@brief ExistCpp的主要接口，用于向局域网发布服务。
 */

#include "EqPreCompile.h" //QUrl

#include "ExistMessage.pb.h" //exist::ServicePublishMessage::ServiceProtocolType
#include "EqNetworkThread.h" //EqNetworkThread

/*!
 * \brief The ServicePublisher class 这个类用于向局域网中发布服务。这是ExistCpp的主要接口。
 */
class  EXIST_QT_EXPORT ServicePublisher:public QObject
{
    Q_OBJECT

signals:
    /*!
     * \brief shoudCastData 信号，应当发送数据。
     * \param datagram 数据包。
     * \internal
     */
    void shoudCastData(string datagram);

    /*!
     * \brief serviceDiscovered 信号，发现了服务。
     * \param serviceName 服务名字。
     * \param servicePort 服务端口号。
     * \param serviceProtocolType 服务的协议类型。
     * \param address 服务的主机地址。
     */
    void serviceDiscovered(QString serviceName,quint16 servicePort,ServicePublishMessage::ServiceProtocolType serviceProtocolType,string address);

public:
    void quit(); //!<退出。
    ~ServicePublisher(); //!<析构函数。
    ServicePublisher(); //!<默认构造函数。
    void publishService(QString LanServiceName,quint16 LanServicePort,exist::ServicePublishMessage::ServiceProtocolType LanServiceProtocolType); //!<发布服务。

private:
    EqNetworkThread networkThread; //!<网络线程。
    void reportServiceDiscovered(ExistMessage existMessage, string address); //!<报告，发现了服务。
    void processDatagram(char *datagram, int datagramLength, QHostAddress senderAddress); //!<处理数据包。
    void connectSignals(); //!<连接信号槽。
    void listen(); //!<监听。
    const quint16 MulticastPort=16574; //!<组播组的端口号。
    void startNetworkThread(); //!<启动网络线程。
}; //class EXISTQTSHARED_EXPORT GameInfo:public QObject

#endif // GAMEINFO_H
